/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "globus_gridftp_server_s3_base.h"

const uint64_t                          GLOBUS_S3_MULTIPART_MAXIMUM
                                      = UINT64_C(5000000000000);
const uint64_t                          GLOBUS_S3_MULTIPART_MINIMUM
                                      = UINT64_C(500000000);
const uint64_t                          GLOBUS_S3_MULTIPART_DEFAULT_SIZE
                                      = UINT64_C(500000000);

/**
 * @file recv.c
 * Implementation of the s3 DSI Receive
 */

/**
 * @brief recv
 * @details
 * This interface function is called when the client requests that a
 * file be transfered to the server.
 *
 * To receive a file the following functions will be used in roughly
 * the following order.  They are described in more detail with the
 * gridftp server documentation.
 *
 * 1. globus_gridftp_server_begin_transfer();
 * 2. globus_gridftp_server_register_read();
 * 3. globus_gridftp_server_finished_transfer();
 *
 */
void
globus_i_s3_recv(
    globus_gfs_operation_t              op,
    globus_gfs_transfer_info_t         *transfer_info,
    void                               *user_arg)
{
    globus_i_s3_handle_t               *s3_handle = user_arg;
    globus_result_t                     result = GLOBUS_SUCCESS;
    uint64_t                            part_size;

    GlobusS3Enter();

    assert(transfer_info != NULL);
    assert(s3_handle != NULL);

    GlobusS3Info("recv op=%p pathname=%s\n", 
                (void *) op,
                transfer_info->pathname);

    // TODO: Missing API for determining part size, use default 500MB
    // which can handle any file up to the s3 maximum.
    part_size = GLOBUS_S3_MULTIPART_DEFAULT_SIZE;

    if (transfer_info->truncate )
    {
        // If this transfer sets the truncate flag, we assume its a new
        // transfer, and can determine whether to use single- or multi-part
        // uploads.
        if (transfer_info->alloc_size > GLOBUS_S3_MULTIPART_MAXIMUM)
        {
            /* Too large for multipart transfer */
            result = GlobusGFSErrorGeneric("File upload too large");
            goto size_error;
        }
        else if (part_size < GLOBUS_S3_MULTIPART_MINIMUM)
        {
            // Only if client expressed a desired multipart upload part
            // size
            result = GlobusGFSErrorGeneric("Multipart upload part size too small");
            goto size_error;
        }
        else if (transfer_info->alloc_size > GLOBUS_S3_MULTIPART_DEFAULT_SIZE)
        {
            // Start a new multipart upload
            result = globus_i_s3_recv_multipart(op, transfer_info, transfer_info->alloc_size, part_size, false, user_arg);
        }
        else
        {
            // Start a new single part upload
            result = globus_i_s3_recv_single(op, transfer_info, user_arg);
        }
    }
    else
    {
        // TODO: Missing API for determining the whole file size.

        // We assume the client is restarting (truncate=false), but don't know
        // the file size, so we trust they will send us the rest of the
        // transfer and we'll be able to complete it.
        result = globus_i_s3_recv_multipart(op, transfer_info, (uint64_t) -1, part_size, true, user_arg);
    }

    if (result != GLOBUS_SUCCESS)
    {
size_error:
        globus_gridftp_server_finished_transfer( 
                op,
                result);
    }
}
/* globus_i_s3_recv() */
// vim: ft=c:
